@@ -311,7 +311,7 @@ int mbedtls_aria_setkey_enc(mbedtls_aria_context *ctx,
     }
 
     i = (keybits - 128) >> 6;
-    ctx->nr = 12 + 2 * i;
+    ctx->nr = (unsigned char) (12 + 2 * i);
 
     aria_fo_xor(w[1], w[0], rc[i], w[1]);
     i = i < 2 ? i + 1 : 0;
