@@ -1,17 +1,21 @@
 library(nanonext)
-nanotest <- function(x) invisible(x || stop("is not TRUE when expected to be TRUE"))
-nanotestw <- function(x) invisible(suppressWarnings(x) || stop("is not TRUE when expected to be TRUE"))
-nanotestz <- function(x) invisible(x == 0L || stop("does not equal 0L as expected"))
-nanotesti <- function(a, b) invisible(identical(a, b) || stop("the arguments are not identical as expected"))
-nanotestaio <- function(x) invisible(is_aio(x) || stop("is not of class 'recvAio' or 'sendAio' as expected"))
-nanotestnano <- function(x) invisible(is_nano(x) || stop("does not inherit from class 'nano' as expected"))
-nanotestn <- function(x) invisible(is.null(x) || stop("is not NULL when expected to be NULL"))
+nanotest <- function(x) invisible(isTRUE(x) || {print(x); stop("the above was returned instead of TRUE")})
+nanotestw <- function(x) invisible(suppressWarnings(isTRUE(x)) || {print(x); stop("the above was returned instead of TRUE")})
+nanotestz <- function(x) invisible(x == 0L || {print(x); stop("the above was returned instead of 0L")})
+nanotesti <- function(a, b) invisible(identical(a, b) || {print(a); print(b); stop("the above arguments were not identical")})
+nanotestaio <- function(x) invisible(is_aio(x) || {print(x); stop("the above was returned instead of an Aio object")})
+nanotestnano <- function(x) invisible(is_nano(x) || {print(x); stop("the above was returned instead of an object of class 'nano'")})
+nanotestn <- function(x) invisible(is.null(x) || {print(x); stop("the above was returned instead of NULL")})
 nanotestnn <- function(x) invisible(!is.null(x) || stop("is NULL when expected to be not NULL"))
-nanotestp <- function(x) invisible(is.character(capture.output(print(x))) || stop("print output of expression cannot be captured as a character value"))
-nanotestxp <- function(x) invisible(typeof(x) == "externalptr" || stop("is not of type 'externalptr' as expected"))
-nanotesterr <- function(x, e = "")
-  invisible(grepl(e, tryCatch(x, error = identity)[["message"]], fixed = TRUE) || stop("expected error message '", e, "' not generated"))
+nanotestp <- function(x) invisible(is.character(capture.output(print(x))) || stop("print output of expression could not be captured as a character value"))
+nanotestxp <- function(x) invisible(typeof(x) == "externalptr" || {print(x); stop("the above was returned instead of an external pointer")})
+nanotesterr <- function(x, e = "") {
+  x <- tryCatch(x, error = identity)
+  inherits(x, "error") && grepl(e, x[["message"]], fixed = TRUE) || stop("expected error message containing '", e, "' was not generated")
+  invisible(TRUE)
+}
 later <- requireNamespace("later", quietly = TRUE)
+promises <- requireNamespace("promises", quietly = TRUE)
 
 nng_version()
 nanotestnano(n <- nano("req", listen = "inproc://nanonext", autostart = FALSE))
@@ -25,7 +29,7 @@ nanotestn(n$newmethod)
 nanotest(is.integer(attr(n$socket, "id")))
 nanotest(n$socket$state == "opened")
 nanotest(n$socket$protocol == "req")
-nanotest(n$send("not ready", mode = "next") == 8L)
+nanotest(n$send("not ready", mode = "serial") == 8L)
 nanotest(n$recv() == 11L)
 nanotestnano(n$opt("recv-size-max", 8192))
 nanotest(n$opt("recv-size-max") == 8192L)
@@ -78,7 +82,7 @@ nanotestz(n1$send(c("test", "", "spec"), mode = "raw", block = 500))
 nanotesti(n$recv("character", block = 500), c("test", "", "spec"))
 nanotestz(n$send(1:5, mode = "r"))
 nanotest(length(n1$recv("int", block = 500)) == 5L)
-nanotestaio(saio <- n1$send_aio(paste(replicate(5, random(1e3L)), collapse = ""), mode = 3L, timeout = 900))
+nanotestaio(saio <- n1$send_aio(paste(replicate(5, random(1e3L)), collapse = ""), mode = 1L, timeout = 900))
 nanotestp(saio)
 nanotestaio(call_aio(saio))
 nanotestz(saio$result)
@@ -196,6 +200,17 @@ nanotesterr(req$opt("false", "false"), "supported")
 nanotesterr(req$opt("false", NULL), "supported")
 nanotesterr(req$opt("false", TRUE), "supported")
 nanotesterr(req$opt("false", list()), "type")
+
+nanotestaio(r <- recv_aio(rep, timeout = 500))
+nanotestz(req$send("", block = 500))
+nanotestp(p <- tryCatch(collect_pipe(r), error = function(e) NULL))
+nanotest(is_nano(p) || is.null(p))
+nanotest(.mark())
+nanotestaio(r <- send_aio(if (is.null(p)) rep else p, "", timeout = 500))
+nanotesterr(collect_pipe(r), "valid")
+nanotest(req$recv(mode = 8L, block = 500)[4L] == 1L)
+nanotest(!.mark(FALSE))
+
 nanotestnano(ctx <- context(rep))
 nanotestp(ctx)
 nanotestaio(csaio <- req$send_aio(data.frame(), mode = "seria", timeout = 500))
@@ -214,7 +229,7 @@ nanotestz(req$send(c(TRUE, FALSE, TRUE), mode = 2L, block = 500))
 nanotestaio(msg <- recv_aio(ctx, mode = 6L, timeout = 500))
 nanotest(is.logical(msg[]))
 nanotesti(collect_aio(msg), collect_aio_(msg))
-nanotestaio(err <- send_aio(ctx, msg[["data"]], mode = "next"))
+nanotestaio(err <- send_aio(ctx, msg[["data"]], mode = "serial"))
 nanotestn(stop_aio(err))
 nanotestaio(err <- send_aio(ctx, "test"))
 nanotest(is_error_value(call_aio(err)$result))
@@ -231,26 +246,32 @@ nanotesti(req$recv(mode = 8L, block = 500), as.raw(1L))
 nanotestaio(rek <- request(req$context, c(1+3i, 4+2i), send_mode = 2L, recv_mode = "complex", timeout = 500))
 nanotestz(reply(ctx, execute = identity, recv_mode = 3L, send_mode = "ra", timeout = 500))
 nanotest(is.complex(call_aio(rek)[["data"]]))
-nanotestaio(rek <- request(req$context, c(1+3i, 4+2i), send_mode = "next", recv_mode = "serial", timeout = 500))
-nanotestz(reply(ctx, execute = identity, recv_mode = 1L, send_mode = 3L, timeout = 500))
+nanotestaio(rek <- request(req$context, c(1+3i, 4+2i), send_mode = "serial", recv_mode = "serial", timeout = 500))
+nanotestz(reply(ctx, execute = identity, recv_mode = 1L, send_mode = 1L, timeout = 500))
 nanotest(is.complex(call_aio(rek)[["data"]]))
 
-nanotest(is.pairlist(nxt <- next_config(refhook = list(function(x) raw(1L), as.integer), class = "custom", vec = FALSE)))
-nanotest(length(nxt) == 2L)
-nanotest(is.function(nxt[[1L]]))
+nanotest(is.list(cfg <- serial_config(class = "custom", sfunc = function(x) raw(1L), ufunc = as.integer, vec = FALSE)))
+nanotest(length(cfg) == 4L)
+nanotest(is.function(cfg[[2L]]))
+opt(req$socket, "serial") <- cfg
+opt(rep, "serial") <- cfg
 custom <- list(`class<-`(new.env(), "custom"), new.env())
-nanotestz(req$send(custom, mode = 3L, block = 500))
+nanotestz(send(req$socket, custom, mode = "serial", block = 500))
 nanotest(is.integer(recv(rep, block = 500)[[1L]]))
-nanotest(is.pairlist(nxt <- next_config(refhook = list(function(x) list(x = as.raw(1L)), function(x) list(x = new.env())), class = "custom", vec = TRUE)))
-nanotestz(send(rep, custom, mode = 3L, block = 500))
-nanotest(is.list(req$recv(mode = 1L, block = 500)))
-nanotesti(next_config(nxt), nxt)
-nanotesti(next_config(as.pairlist(c("a", "b", "c"))), nxt)
-nanotest(is.list(next_config(NULL, mark = TRUE)))
-nanotestn(unlist(next_config()))
-nanotesterr(next_config(NULL, class = 1L), "must be a character string")
+cfg <- serial_config("custom", function(x) as.raw(length(x)), function(x) lapply(seq_len(as.integer(x)), new.env), vec = TRUE)
+opt(req$socket, "serial") <- cfg
+opt(rep, "serial") <- cfg
+nanotestz(send(rep, custom, block = 500))
+nanotest(is.list(recv(req$socket, mode = 1L, block = 500)))
+opt(req$socket, "serial") <- list()
+opt(rep, "serial") <- list()
+nanotesterr(serial_config("custom", "func1", "func2"), "must be functions")
+nanotesterr(opt(rep, "wrong") <- cfg, "not supported")
+nanotesterr(opt(rep, "serial") <- pairlist(a = 1L), "not supported")
+nanotesterr(opt(rep, "serial") <- list("wrong"), "Invalid argument")
+nanotestw(is.null(next_config()))
 
-nanotestaio(cs <- request(req$context, "test", send_mode = "next", cv = cv, timeout = 500))
+nanotestaio(cs <- request(req$context, "test", send_mode = "serial", cv = cv, timeout = 500))
 if (later) nanotestaio(set_promise_context(cs, environment()))
 nanotestnn(cs$data)
 nanotest(typeof(ctxn <- .context(rep)) == "externalptr")
@@ -261,7 +282,7 @@ nanotest(typeof(ctxn <- .context(rep)) == "externalptr")
 nanotestaio(cs <- request(.context(req$socket), data = TRUE, cv = NA))
 nanotestnn(cs$data)
 nanotest(recv(ctxn, block = 500))
-nanotestz(send(ctxn, TRUE, mode = 3L, block = 500))
+nanotestz(send(ctxn, TRUE, mode = 1L, block = 500))
 nanotestz(reap(ctxn))
 nanotest(reap(ctxn) == 7L)
 nanotestz(pipe_notify(rep, cv, add = TRUE, flag = TRUE))
@@ -277,6 +298,7 @@ nanotesterr(until_(err, 10), "valid")
 nanotesterr(cv_value(err), "valid")
 nanotesterr(cv_reset(err), "valid")
 nanotesterr(cv_signal(err), "valid")
+nanotesterr(collect_pipe(err), "valid")
 nanotesterr(pipe_notify(err, cv), "valid Socket")
 nanotesterr(pipe_notify(rep, err), "valid Condition Variable")
 nanotesterr(pipe_notify(rep, cv, err), "valid Condition Variable")
@@ -289,6 +311,8 @@ nanotestxp(rep$dialer[[1L]])
 nanotestz(close(ctx))
 nanotestw(close(ctx) == 7L)
 nanotestz(close(rep))
+nanotest(if (is.null(p)) TRUE else reap(p) == 12L)
+nanotestw(if (is.null(p)) TRUE else close(p) == 12L)
 
 nanotestnano(pub <- nano("pub", listen = "inproc://ps"))
 nanotestnano(sub <- nano("sub", dial = "inproc://ps", autostart = NA))
@@ -346,6 +370,7 @@ nanotest(inherits(bus <- socket(protocol = "bus"), "nanoSocket"))
 nanotest(inherits(push <- socket(protocol = "push"), "nanoSocket"))
 nanotest(inherits(pull <- socket(protocol = "pull"), "nanoSocket"))
 nanotest(inherits(pair <- socket(protocol = "pair"), "nanoSocket"))
+nanotest(inherits(poly <- socket(protocol = "poly"), "nanoSocket"))
 nanotestnano(bus)
 nanotestw(listen(bus, url = "test") == 3L)
 nanotestw(dial(bus, url = "test") == 3L)
@@ -354,8 +379,9 @@ nanotesterr(dial(bus, url = "tls+tcp://localhost/:0", tls = "wrong"), "valid TLS
 nanotestz(close(bus))
 nanotestw(close(bus) == 7L)
 nanotestz(close(push))
-nanotestz(reap(pull))
+nanotestz(close(pull))
 nanotestz(reap(pair))
+nanotestz(reap(poly))
 nanotesterr(socket(protocol = "newprotocol"), "protocol")
 nanotesterr(socket(dial = "test"), "argument")
 nanotesterr(socket(listen = "test"), "argument")
@@ -365,32 +391,33 @@ nanotestnn(ncurl("http://www.cam.ac.uk/", follow = FALSE, response = "date"))
 nanotestnn(ncurl("http://www.cam.ac.uk/", follow = TRUE))
 nanotestnn(ncurl("http://postman-echo.com/post", convert = FALSE, method = "POST", headers = c(`Content-Type` = "text/plain"), data = "test", response = c("Date", "Server"), timeout = 3000))
 nanotest(is_error_value(ncurl("http")$data))
-haio <- ncurl_aio("http://trin.cam.ac.uk/")
-nanotest(length(call_aio(haio)$headers))
-haio <- ncurl_aio("https://trin.cam.ac.uk/", convert = FALSE, response = "server")
-nanotestnn(haio$status)
+haio <- ncurl_aio("http://example.com/")
 nanotest(is.integer(call_aio(haio)$status))
+haio <- ncurl_aio("https://example.com/", convert = FALSE, response = "server")
+nanotestnn(haio$status)
+if (call_aio(haio)$status == 200L) nanotestnn(haio$headers)
 put1 <- ncurl_aio("http://postman-echo.com/put", method = "PUT", headers = c(Authorization = "Bearer token"), data = "test", response = c("Date", "server"), timeout = 3000L)
 nanotestp(put1)
 nanotest(is.integer(call_aio_(put1)$status))
-nanotestnn(put1$headers)
-nanotestnn(put1$data)
+if (put1$status == 200L) nanotestnn(put1$headers)
+if (put1$status == 200L) nanotestnn(put1$data)
 nanotestn(stop_aio(put1))
 haio <- ncurl_aio("https://i.i")
 nanotest(is_error_value(call_aio(haio)$data))
 nanotestp(haio$data)
 ncaio <- ncurl_aio("https://shikokuchuo.net/nanonext/reference/figures/logo.png")
-nanotest(is.raw(call_aio(ncaio)$data) || is_error_value(ncaio$data))
+if (call_aio(ncaio)$status == 200L) nanotest(is.raw(ncaio$data))
 nanotest(is_error_value(ncurl_aio("http")$data))
 sess <- ncurl_session("https://postman-echo.com/post", method = "POST", headers = c(`Content-Type` = "text/plain"), data = "test", response = c("date", "Server"), timeout = 3000L)
-nanotestnn(sess)
-nanotest(is_error_value(sess) || length(transact(sess)) == 3L)
-nanotest(is_error_value(sess) || close(sess) == 0L)
+nanotest(is_ncurl_session(sess) || is_error_value(sess))
+if (is_ncurl_session(sess)) nanotest(length(transact(sess)) == 3L)
+if (is_ncurl_session(sess)) nanotest(close(sess) == 0L)
+if (is_ncurl_session(sess)) nanotestw(close(sess) == 7L)
 sess <- ncurl_session("https://postman-echo.com/post", convert = FALSE, method = "POST", headers = c(`Content-Type` = "text/plain"), timeout = 3000)
-nanotestnn(sess)
-nanotest(is_error_value(sess) || length(transact(sess)) == 3L)
-nanotest(is_error_value(sess) || close(sess) == 0L)
-nanotesterr(transact(sess), "ncurlSession")
+nanotest(is_ncurl_session(sess) || is_error_value(sess))
+if (is_ncurl_session(sess)) nanotest(length(transact(sess)) == 3L)
+if (is_ncurl_session(sess)) nanotest(close(sess) == 0L)
+if (is_ncurl_session(sess)) nanotest(transact(sess)$data == 7L)
 nanotestw(is_error_value(ncurl_session("https://i")))
 nanotesterr(ncurl_aio("https://", tls = "wrong"), "valid TLS")
 nanotesterr(ncurl("https://www.cam.ac.uk/", tls = "wrong"), "valid TLS")
@@ -405,18 +432,21 @@ nanotesterr(stream(listen = "inproc://notsup"), "Not supported")
 nanotesterr(stream(listen = "errorValue3", tls = "wrong"), "valid TLS")
 nanotesterr(stream(), "specify")
 
-nanotest(is.character(ver <- nng_version()) && length(ver) == 2L)
+nanotest(is.character(ver <- nng_version()))
+nanotest(length(ver) == 2L)
 nanotest(nng_error(8L) == "8 | Try again")
 nanotest(is_nul_byte(as.raw(0L)))
 nanotest(!is_nul_byte(NULL))
 nanotest(!is_error_value(1L))
 nanotesterr(messenger("invalidURL"), "argument")
-nanotest(is.raw(md5 <- nanonext:::md5_object("secret base")) && length(md5) == 32L)
+nanotest(is.raw(md5 <- nanonext:::md5_object("secret base")))
+nanotest(length(md5) == 32L)
 nanotest(is.double(mclock()))
 nanotestn(msleep(1L))
 nanotestn(msleep(1))
 nanotestn(msleep("a"))
-nanotest(is.character(urlp <- parse_url("://")) && length(urlp) == 10L)
+nanotest(is.character(urlp <- parse_url("://")))
+nanotest(length(urlp) == 10L)
 nanotest(all(nzchar(parse_url("wss://use:r@[::1]/path?q=1#name"))))
 nanotest(is.character(random()))
 nanotest(nchar(random(2)) == 4L)
@@ -429,7 +459,7 @@ for (i in c(100:103, 200:208, 226, 300:308, 400:426, 428:431, 451, 500:511))
   nanotest(is.character(status_code(i)))
 
 s <- tryCatch(stream(dial = "wss://echo.websocket.events/", textframes = TRUE), error = function(e) NULL)
-if (length(s)) {
+is_nano(s) && {
   nanotestnn(recv(s, block = 500L))
   nanotest(is.character(opt(s, "ws:response-headers")))
   nanotesterr(opt(s, "ws:request-headers") <- "test\n", 24)
@@ -439,20 +469,21 @@ if (length(s)) {
   nanotestnn(recv(s, mode = 9L, block = 100))
   nanotest(is.integer(send(s, 2L, block = 500)))
   nanotestaio(sr <- recv_aio(s, mode = "i", timeout = 500L, n = 8192L))
-  nanotest(length(call_aio(sr)[["data"]]))
+  nanotestnn(call_aio(sr)[["data"]])
   nanotestn(stop_aio(sr))
   nanotestaio(ss <- send_aio(s, "async", timeout = 500L))
   nanotest(is.integer(ss[]))
   nanotestn(stop_aio(ss))
   nanotest(is.integer(send(s, 12.56, mode = "raw", block = 500L)))
   nanotestaio(sr <- recv_aio(s, mode = "double", timeout = 500L, cv = cv))
-  nanotest(length(call_aio_(sr)[["data"]]))
+  nanotestnn(call_aio_(sr)[["data"]])
   nanotest(cv_value(cv) > 0L)
   nanotest(is.character(opt(s, "ws:request-headers")))
   nanotestnn(opt(s, "tcp-nodelay") <- FALSE)
   nanotesterr(recv(s, mode = "none", block = FALSE), "mode")
   nanotesterr(recv(s, mode = "c", block = FALSE), "mode")
   nanotesterr(opt(s, "none"), "supported")
+  nanotesterr(`opt<-`(s, "none", list()), "supported")
   nanotestp(s)
   nanotest(is.integer(close(s)))
 }
@@ -511,6 +542,7 @@ nanotesterr(send(fakestream, "test"), "valid")
 nanotesterr(send_aio(fakestream, "test"), "valid")
 nanotesterr(recv(fakestream), "valid")
 nanotesterr(recv_aio(fakestream), "valid")
+nanotesterr(collect_pipe(fakestream), "valid")
 nanotesterr(opt(fakestream, name = "test") <- "test", "valid")
 nanotesterr(opt(fakestream, name = "test"), "valid")
 nanotesterr(close(fakestream), "active Stream")
@@ -520,11 +552,15 @@ nanotesterr(close(fakedial), "valid Dialer")
 fakelist <- `class<-`("test", "nanoListener")
 nanotesterr(start(fakelist), "valid Listener")
 nanotesterr(close(fakelist), "valid Listener")
+fakepipe <- `class<-`("test", "nanoPipe")
+nanotesterr(close(fakepipe), "valid Pipe")
 unres <- `class<-`(NA, "unresolvedValue")
 nanotest(!unresolved(unres))
 nanotestp(unres)
-nanotest(is.logical(unres <- unresolved(list("a", "b"))) && length(unres) == 1L)
-nanotest(is.integer(unres <- .unresolved(list("a", "b"))) && length(unres) == 1L)
+nanotest(is.logical(unres <- unresolved(list("a", "b"))))
+nanotest(length(unres) == 1L)
+nanotest(is.integer(unres <- .unresolved(list("a", "b"))))
+nanotest(length(unres) == 1L)
 nanotesti(call_aio("a"), "a")
 nanotesti(call_aio_("a"), "a")
 nanotesterr(collect_aio_("a"), "object is not an Aio or list of Aios")
@@ -546,7 +582,8 @@ nanotest(test_tls(pem = pem))
 nanotesterr(tls_config(client = c(pem, pem)), "Cryptographic error")
 nanotesterr(tls_config(server = c(pem, pem)), "Cryptographic error")
 nanotest(is.list(cert <- write_cert(cn = "127.0.0.1")))
-nanotest(length(cert) == 2L && is.character(cert[[1L]]))
+nanotest(length(cert) == 2L)
+nanotest(is.character(cert[[1L]]))
 nanotesti(names(cert), c("server", "client"))
 nanotestxp(tls <- tls_config(client = cert$client))
 nanotest(inherits(tls, "tlsConfig"))
@@ -554,21 +591,21 @@ nanotestp(tls)
 nanotest(is_error_value(ncurl("https://www.cam.ac.uk/", tls = tls)$status))
 nanotest(is_error_value(call_aio(ncurl_aio("https://www.cam.ac.uk/", tls = tls))$data))
 nanotesterr(ncurl_session("https://www.cam.ac.uk/", tls = cert$client), "not a valid TLS")
-nanotestw(typeof(b <- ncurl_session("https://www.cam.ac.uk/", tls = tls)) == "externalptr" || is_error_value(b))
-nanotest(if (!is_error_value(b)) is_error_value(transact(b)[["headers"]]) else TRUE)
+sess <- ncurl_session("https://www.cam.ac.uk/", tls = tls)
+nanotest(is_ncurl_session(sess) || is_error_value(sess))
+if (is_ncurl_session(sess)) nanotest(is_error_value(transact(sess)[["headers"]]))
 nanotestxp(s <- socket(listen = "tls+tcp://127.0.0.1:5556", tls = tls_config(server = cert$server)))
 nanotestxp(s1 <- socket(dial = "tls+tcp://127.0.0.1:5556", tls = tls))
 nanotestw(dial(s, url = "tls+tcp://.", tls = tls, error = FALSE) > 0)
 nanotestw(listen(s, url = "tls+tcp://.", tls = tls, error = FALSE) > 0)
 nanotestz(close(s1))
 nanotestz(close(s))
-if (requireNamespace("promises", quietly = TRUE)) {
-  nanotestaio(n <- ncurl_aio("https://postman-echo.com/get"))
-  nanotest(tryCatch(promises::is.promise(promises::then(n, cat)), error = function(e) TRUE))
-  nanotest(promises::is.promising(call_aio(n)))
-  nanotest(promises::is.promise(promises::as.promise(call_aio(ncurl_aio("https://postman-echo.com/get")))))
-  later::run_now()
-}
+nanotest(!identical(get0(".Random.seed"), {.advance(); .Random.seed}))
+if (promises) nanotestaio(n <- ncurl_aio("https://postman-echo.com/get"))
+if (promises) nanotest(tryCatch(promises::is.promise(promises::then(n, cat)), error = function(e) TRUE))
+if (promises) nanotest(promises::is.promising(call_aio(n)))
+if (promises) nanotest(promises::is.promise(promises::as.promise(call_aio(ncurl_aio("https://postman-echo.com/get")))))
+if (promises) later::run_now()
 if (Sys.info()[["sysname"]] == "Linux") {
   rm(list = ls())
   gc()
